MODULE dbgmonitor; 

SFR PSW; TFR; P3; DP3; S0BG; S0RIC; S0TIC; S0CON; 

CONST Clock = 24000; (* in kHz *)
      Baudrate = 38400;  
      CPsv = $F600; (* Startwert CP *)
      
    PROCEDURE* IniASC0;
    VAR i: INTEGER;
    BEGIN i := DP3; DP3 := i ORB 1024; (* P3.10 = TxD, output *)
      i := P3; P3 := i ORB 1024; (* P3.10 = TxD = 1 *)      
      i := Baudrate DIV 100; i := Clock DIV i;
      i := i * 5; i := i DIV 16; i := i - 1 ;
      S0BG := i; (* Baudrate gesetzt *)
      S0RIC := %01110111; (* hchster Interrupt in Level und Group *)
      S0TIC := 0; (* kein Interrupt hierdurch *)
      S0CON := $8011; (* 8N1 *)
    END IniASC0;

    PROCEDURE* Send(w1, w2: INTEGER);
    BEGIN
      VERBATIM
	mov	R2, R15
	shr	R2, #8
	mov	S0TBUF, R2
      dbgmonitorsend0:
	jnb	S0TIR, dbgmonitorsend0
	bclr	S0TIR
	mov	R2, R15
	and	R2, #255
	mov	S0TBUF, R2
      dbgmonitorsend1:
	jnb	S0TIR, dbgmonitorsend1	
	bclr	S0TIR
	mov	R2, R14
	shr	R2, #8
	mov	S0TBUF, R2
      dbgmonitorsend2:
	jnb	S0TIR, dbgmonitorsend2
	bclr	S0TIR
	mov	R2, R14
	and	R2, #255
	mov	S0TBUF, R2
      dbgmonitorsend3:
	jnb	S0TIR, dbgmonitorsend3
	bclr	S0TIR
      END_VERBATIM
    END Send;

    PROCEDURE* Start;
    VAR i : INTEGER;
    BEGIN
      VERBATIM
	pop	R15	; returnaddr.
	pop	CP	; 
	bset	IEN
	reti
      END_VERBATIM
    END Start;    
    
    PROCEDURE* SetStart;
    VAR seg, adr, i:INTEGER;
    BEGIN Read(seg); 
      Read(adr); adr := adr * 256;
      Read(i); adr := adr + i;
      VERBATIM
	mov	R13, SP
	exts	#0, #2
	mov	[R13+#6], R14	; adr = IP
	mov	[R13+#8], R15	; seg = CSP
      END_VERBATIM
    END SetStart;
    
    PROCEDURE* MakeStack;
    VAR i, d1, d2, reta, npsw, dcp: INTEGER;
    BEGIN i := 0;
      WHILE i # 65 DO (* 65 = A *)
	Read(i); END;
      Read(d1);
      Read(i); d2 := i * 256;
      Read(i); d2 := d2 + i;
      i := CPsv + 96; (* 3 Registerstz fr den Monitor *)
      npsw := 0;
      VERBATIM
	pop	R12		; reta := returnaddr.
	pop	R10		; remove CP 
	push	R11		; new PSW = 0
	push	R14		; d1 = CSP
	push	R13		; d2 = IP
	push	R15		; i = CP
	mov	[R15], R0	; save framepointer
	push	R12		; returnaddress
      END_VERBATIM
    END MakeStack;
    
    PROCEDURE* SendRegs;
    VAR p, reg1, reg2, i, reta: INTEGER;
    BEGIN
      VERBATIM
	pop	R11		; reta := returnaddr.
	pop	R15		; p := CP
	push	R15		
	push	R11		; Stack o.k.
      END_VERBATIM;      
      FOR i := 0 TO 7 DO
	VERBATIM
	  exts	#0, #2
	  mov	R14, [R15]	; reg1 := R(x)
	  mov	R13, [R15+#2]	; reg2 := R(x+1)
	END_VERBATIM
	p := p + 4; Send(reg1,reg2);
      END;    
    END SendRegs;
    
    PROCEDURE* SendMem;
    VAR seg, adr, v1, v2, i: INTEGER;
    BEGIN Read(seg); 
      Read(adr); adr := adr * 256;
      Read(i); adr := adr + i;
      FOR i := 0 TO 7 DO
	VERBATIM
	  exts	R15, #2
	  mov	R13, [R14]
	  mov	R12, [R14+#2]
	END_VERBATIM
	adr := adr + 4;
	Send(v1,v2);
      END;
    END SendMem;
    
    PROCEDURE* SendPSW;
    VAR d1, d2, i: INTEGER;
    BEGIN d1 := 80 * 256; (* P *)
      VERBATIM
        mov	R13, SP
        mov	R14, [R13+#8]	; d2 := PSW
      END_VERBATIM
      Send(d1, d2);
    END SendPSW;
    
    PROCEDURE* SetBreak(VAR org:INTEGER);
    VAR seg, adr, val, undop: INTEGER;
    BEGIN undop := $4444; (* undefined opcode *)
      Read(seg);
      Read(adr); adr := adr * 256;
      Read(val); adr := adr + val;
      VERBATIM
	exts	R14, #2
	mov	R12, [R13]	; val := original
	mov	[R13], R11
      END_VERBATIM
      org := val;
    END SetBreak;
    
    PROCEDURE* ResBreak;
    VAR seg, adr, val, i: INTEGER;
    BEGIN Read(seg); Read(adr); adr := adr * 256;
      Read(i); adr := adr + i; Read(val); 
      val := val * 256; Read(i); val := val + i;
      VERBATIM
	exts	R15, #1
	mov	[R14], R13
      END_VERBATIM
    END ResBreak;

    PROCEDURE* control;
    VAR i, d1, d2, d3, d4: INTEGER;
    BEGIN 
      i := CPsv; 
      VERBATIM
	mov	CP, R15		; CP := i
	nop
      END_VERBATIM
      IniASC0;   
      i := PSW; i := i ANDB 64; (* USR0 maskieren *)
      IF i # 0 THEN (* Reset *)	
	PSW := 0; (* USR0 lschen *)
	d1 := 82 * 256; d2 := 0; Send(d1,d2);
	MakeStack;
      ELSE
	i := TFR; (* woher kommen wir? *)
	TFR := 0;
	IF i >= $8000 THEN (* NMI *)
	  d1 := 78; (* N *)
	ELSIF i = 128 THEN (* UNDOP *)
	  d1 := 73; (* I *)
	ELSIF i = 4 THEN (* ILLOPA *)
	  d1 := 65; (* A *)
	ELSIF i = 2 THEN (* ILLINA *)
	  d1 := 77; (* M *)
	ELSIF i = $4000 THEN (* STKOF *)
	  d1 := 67; (* C *)
	ELSE (* ASC- Byte *)
	  d1 := 83; (* S *)       
	END;	
	d1 := d1 * 256;
	VERBATIM ; Adresse laden, Stack wiederherstellen
	  pop	R13		; d2 := CP
	  pop	R12		; d3 := IP
	  pop	R15		; i := CSP
	  push	R15		; CSP
	  push	R12		; IP
	  push	R13		; CP
	END_VERBATIM
	i := i ANDB 255; d1 := d1 + i;
	Send(d1, d3);
      END;      
      REPEAT (* Endlosschleofe *)
	Read(i);
	IF i = 82 THEN (* "R": Reset Target *)
	  VERBATIM
	    jmps	0,0
	  END_VERBATIM
	ELSIF i = 65 THEN (* "A": Startadresse setzen *)
	  SetStart;
	ELSIF i = 83 THEN (* "S": Starte Anwendung *)
	  Start;
	ELSIF i = 66 THEN (* "B": Breakpoint setzen *)
	  SetBreak(d2); d1 := 66 * 256;
	  Send(d1,d2);
	ELSIF i = 69 THEN (* "E": Breakpoint ersetzen *)
	  ResBreak;
	ELSIF i = 86 THEN (* "V": sende Werte *)
	  Read(i);
	  IF i = 77 THEN (* "M": Speicherwerte *)
	    SendMem;
	  ELSIF i = 82 THEN (* "R": Registerwerte *)
	    SendRegs;
	  END;
	ELSIF i = 80 THEN (* "P": PSW senden *)
	  SendPSW;
	END;
      UNTIL 1 > 2;    
    END control;

BEGIN  
    VERBATIM
    bclr	IEN
    push	CP
    jmp		dbgmonitorcontrol
    END_VERBATIM
END dbgmonitor.
